/*
 * File:	DemoRTC.cpp
 *
 * Date:	2012-10-22
 * Company:	DH electronics GmbH
 * Author:	Ludwig Zenz, lzenz@dh-electronics.de
 *
 * Remark:	The following code aims to offer you some help to get started
 * 			with using the RTC device-file.
 */


/*
 * For further RTC documentation and examples have a look at:
 * ->	http://www.kernel.org/doc/Documentation/rtc.txt
 */


/* IO Header */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <linux/rtc.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <time.h>


/* rtc device path */
static const char rtc_path[] = "/dev/rtc0";

int main()
{
	int fd, retval;
	struct tm  tm_time_struct, *tm_gmt_time_struct;
	time_t allsec;

	printf("\nStarting RTC demo ...\n\n");

	/* initialize time_t struct with NULL */
	allsec=time(NULL);

	/* open rtc device-file */
	fd = open(rtc_path, O_RDWR);

	/* Read the RTC time/date (normally Greewich Mean Time)*/
	retval = ioctl(fd, RTC_RD_TIME, &tm_time_struct);
	if (retval == -1) {
		printf("ERROR: RTC_RD_TIME ioctl");
		exit(errno);
	}

	printf("\n\nCurrent RTC date/time is %d-%d-%d, %02d:%02d:%02d.\n",
			tm_time_struct.tm_mday, tm_time_struct.tm_mon + 1, tm_time_struct.tm_year + 1900,
			tm_time_struct.tm_hour, tm_time_struct.tm_min, tm_time_struct.tm_sec);

	/*--------------------------------------------------------------------------------------------*/
	/* Adding 1 hour and set rtc */

	tm_time_struct.tm_hour += 1;

	/* convert localtime to gmt (Greenwich Mean Time (UTC)) */
	allsec=mktime(&tm_time_struct);
	tm_gmt_time_struct = gmtime(&allsec);

	ioctl(fd, RTC_SET_TIME, tm_gmt_time_struct);

	/* convert localtime to gmt (Greenwich Mean Time (UTC)) - in this case not neccessary because we work already with UTC */
	retval = ioctl(fd, RTC_RD_TIME, &tm_time_struct);
	if (retval == -1) {
		printf("ERROR: RTC_RD_TIME ioctl");
		exit(errno);
	}

	printf("\n\nCurrent RTC date/time is %d-%d-%d, %02d:%02d:%02d.\n",
			tm_time_struct.tm_mday, tm_time_struct.tm_mon + 1, tm_time_struct.tm_year + 1900,
			tm_time_struct.tm_hour, tm_time_struct.tm_min, tm_time_struct.tm_sec);

	/*--------------------------------------------------------------------------------------------*/
	/* subtract 1 hour and set rtc */

	tm_time_struct.tm_hour -= 1;

	/* convert localtime to gmt (Greenwich Mean Time (UTC)) - in this case not neccessary because we work already with UTC */
	allsec=mktime(&tm_time_struct);
	tm_gmt_time_struct = gmtime(&allsec);

	ioctl(fd, RTC_SET_TIME, tm_gmt_time_struct);

	/* Read the RTC time/date (normally Greewich Mean Time)*/
	retval = ioctl(fd, RTC_RD_TIME, &tm_time_struct);
	if (retval == -1) {
		printf("ERROR: RTC_RD_TIME ioctl");
		exit(errno);
	}

	printf("\n\nCurrent RTC date/time is %d-%d-%d, %02d:%02d:%02d.\n",
			tm_time_struct.tm_mday, tm_time_struct.tm_mon + 1, tm_time_struct.tm_year + 1900,
			tm_time_struct.tm_hour, tm_time_struct.tm_min, tm_time_struct.tm_sec);

	close(fd);

	return 0;
}

